
(function (document) {

    var ilove3d = true;
    var websocket = null;

    var drawer = null;

    var canvas2D = null;
    var context2D = null;

    var sendingImageRequested = false;

    function initializeWebSocket() {

        websocket = new WebSocketClient();

        websocket.addListener("connected", function () {

            console.log("Connected");
        });

        websocket.addListener("initialize", function () {

            console.log("Initialized");
            document.wsForm.connectButton.disabled = true;
            document.wsForm.disconnectButton.disabled = false;
        });

        websocket.addListener("close", function () {

            console.log("Disconnected");
            document.wsForm.connectButton.disabled = false;
            document.wsForm.disconnectButton.disabled = true;
        });

        websocket.addListener("error", function (evt) {

            console.error(evt);
            websocket.close();
            document.wsForm.connectButton.disabled = false;
            document.wsForm.disconnectButton.disabled = true;
        });

        websocket.addListener("message", function (data, diff) {

            document.getElementById("message").innerHTML = JSON.stringify(data, null, 2);
        });
    }

    function connect() {

	    websocket.open(document.wsForm.url.value);
    }

    function disconnect() {

	    websocket.close();
    }

    function draw2D(context) {

        context.strokeStyle = 'rgb(0, 0, 255)';
        context.fillStyle = 'rgb(255, 0, 0)';
        context.fillRect(140, 80, 50, 120);

        context.strokeStyle = 'rgb(0, 0, 255)';
        context.fillStyle = 'rgb(255, 0, 0)';
        context.strokeRect(270, 120, 100, 50);

        context.arc(250, 250, 160, Math.PI * 1, Math.PI * 2, true);
        context.fill();
    }

    function sendImage() {

        if (ilove3d) {

            context2D.drawImage(drawer.canvas, 0, 0);
        }

        var imageData = context2D.getImageData(0, 0, canvas2D.width, canvas2D.height).data;
        var byteArray = new Uint8Array(imageData);
        websocket.sendByteArray(byteArray.buffer);
    }

    function run() {

        drawer.draw();

        if (sendingImageRequested) {

            sendingImageRequested = false;

            sendImage();
        }

        requestAnimationFrame(run);
    }

    window.onload = (function () {

        initializeWebSocket();

        if (ilove3d) {

            drawer = initWebglDrawer('drawer_canvas');

            canvas2D = document.createElement('canvas');
            canvas2D.width = drawer.canvas.width;
            canvas2D.height = drawer.canvas.height;
            context2D = canvas2D.getContext('2d');
            document.getElementById('imageDrawer').appendChild(canvas2D);
        }
        else {

            canvas2D = document.getElementById('drawer_canvas');
            context2D = canvas2D.getContext('2d');
            draw2D(context2D);
        }

        document.wsForm.disconnectButton.disabled = true;
        //document.wsForm.sendImageButton.disabled = true;

        document.getElementById('connectButton').addEventListener('click', function () {

            connect();
        });

        document.getElementById('disconnectButton').addEventListener('click', function () {

            disconnect();
        });

        document.getElementById('sendImageButton').addEventListener('click', function () {

            sendingImageRequested = true;
        });

        run();
    });

})(document);