// こちらの記事を参考にさせていただきました：「こぴぺするだけ簡単 WebGL + GLSL テンプレート」https://qiita.com/doxas/items/000f2ad6f82c27bd389e

var initWebglDrawer = function (canvasID) {

    var canvas = document.getElementById(canvasID);

    canvas.width = 512;
    canvas.height = 512;
    var gl = canvas.getContext('webgl', { preserveDrawingBuffer: false, antialias: false });

    var program = gl.createProgram();

    function createShader(tagID, shaderType) {

        var shaderCode = document.getElementById(tagID).text;
        var shader = gl.createShader(shaderType);

        gl.shaderSource(shader, shaderCode);
        gl.compileShader(shader);
        gl.attachShader(program, shader);

        return gl.getShaderInfoLog(shader);
    }

    if (!createShader('vs', gl.VERTEX_SHADER) && !createShader('fs', gl.FRAGMENT_SHADER)) {

        gl.linkProgram(program);
    }
    else {

        return null;
    }

    var err = gl.getProgramParameter(program, gl.LINK_STATUS);

    gl.useProgram(program);

    var uniforms = {};
    uniforms.time = gl.getUniformLocation(program, 'time');
    uniforms.resolution = gl.getUniformLocation(program, 'resolution');

    gl.bindBuffer(gl.ARRAY_BUFFER, gl.createBuffer());
    gl.bufferData(gl.ARRAY_BUFFER, new Float32Array([-1, 1, 0, -1, -1, 0, 1, 1, 0, 1, -1, 0]), gl.STATIC_DRAW);

    var aPosition = gl.getAttribLocation(program, 'position');
    gl.enableVertexAttribArray(aPosition);
    gl.vertexAttribPointer(aPosition, 3, gl.FLOAT, false, 0, 0);

    gl.clearColor(0, 0, 0, 1);
    gl.clear(gl.COLOR_BUFFER_BIT);

    var atnimationTimex = 0.0;

    function draw() {

        if (!err) { return; }

        gl.viewport(0, 0, canvas.width, canvas.height);

        atnimationTimex += 0.05;

        gl.clear(gl.COLOR_BUFFER_BIT);

        gl.uniform1f(uniforms.time, atnimationTimex);
        gl.uniform2fv(uniforms.resolution, [canvas.width, canvas.height]);

        gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);
        gl.flush();
    }

    var result = {
        gl: gl,
        canvas: canvas,
        draw: draw
    };

    return result;
};