
WebSocketClient = (function () {

    function WebSocketClient() {

        this.websocket = null;
        this.connected = false;
        this.listeners = {};
    }

    WebSocketClient.prototype.addListener = WebSocketClient.prototype.on = function (event, handler) {

        if (!this.listeners[event]) {

            this.listeners[event] = [handler];
        }
        else {

            this.listeners[event].push(handler);
        }
    };

    WebSocketClient.prototype.close = function () {

        if (this.websocket) {

            this.websocket.close(); // cleared in onclose
        }
    };

    WebSocketClient.prototype.emitEvent = function (event) {

        var handlers = this.listeners[event];

        if (handlers) {

            var args = Array.prototype.slice.call(arguments, 1);

            handlers.forEach(function (handler) {
                handler.apply(null, args);
            });
        }
    };

    WebSocketClient.prototype.open = function (url) {

        if (this.websocket) {
            return;
        }

        var websocket = new WebSocket(url);

        var _this = this;

        websocket.onopen = function () {

            _this.emitEvent("connected");
        };

        websocket.onclose = function () {

            if (_this.connected) {

                _this.emitEvent("close");
            }

            _this.connected = false;
            _this.websocket = null;
        };

        websocket.onerror = function (event) {

            _this.emitEvent("error");
        };

        websocket.onmessage = function (event) {

            try {

                var data = JSON.parse(event.data);
            }
            catch (err) {

                _this.emitEvent("badFormat", event.data);
                return;
            }

            switch (data.type) {

                case "initialize":

                    if (!_this.connected) {

                        _this.connected = true;
                        _this.emitEvent("initialize");
                    }

                    console.log("recieve: initialize " + event.data)

                    _this.sendJson(
                        {
                            "type": "initialized"
                        }
                    );

                    break;

                default:
                    console.log("message: " + event.data)
                    _this.emitEvent("message", data);
                    break;
            }
        };

        this.websocket = websocket;
    };

    WebSocketClient.prototype.removeListener = WebSocketClient.prototype.off = function (event, handler) {

        if (!this.listeners[event]) {
            return;
        }

		var index = this.listeners[event].indexOf(handler);

        if (index !== -1) {

            this.listeners[event].splice(index, 1);
        }
	};

    WebSocketClient.prototype.sendJson = function(jsonObject) {

		var message = JSON.stringify(jsonObject);
		
		this.websocket.send(message);
		
		console.log("send: " + message)
	};

	WebSocketClient.prototype.sendByteArray = function(byteArray) {

		this.websocket.send(byteArray);
		
		console.log("send: binary(" + byteArray.byteLength + ")")
	};
	
	return WebSocketClient;
})();
