﻿
import threading
import os

from wsgiref.simple_server import make_server
from ws4py.websocket import WebSocket
from ws4py.server.wsgirefserver import WSGIServer, WebSocketWSGIRequestHandler
from ws4py.server.wsgiutils import WebSocketWSGIApplication

# Socket event handler class
class WebSocketWSGIApplicationEx(WebSocketWSGIApplication):

	def __init__(self, server, protocols=None, extensions=None, handler_cls=WebSocket):
		super().__init__(protocols, extensions, handler_cls)

		self.server = server
		self.handler = None

	def make_websocket(self, sock, protocols, extensions, environ):

		# **** copy of WebSocketWSGIApplication.make_websocket **** start
		websocket = self.handler_cls(sock, protocols, extensions,
										environ.copy())
		environ['ws4py.websocket'] = websocket
		# **** copy of WebSocketWSGIApplication.make_websocket **** end

		# addtional
		self.handler = websocket
		self.handler.WebSocketServer = self.server
        
		return websocket

class WebSocketEventHundler(WebSocket):

	def __init__(self, sock, protocols, extensions, environ):
		super().__init__(sock, protocols, extensions, environ)

		print("WebSocketEventHundler init")

		self.WebSocketServer = None

	def opened(self):

		print("WebSocketEventHundler opened")
		WebSocketEventHundler.WebSocketServer.sockets.append(self)

		self.WebSocketServer.listener_started()

	def closed(self, code, reason=None):

		print("WebSocketEventHundler closed")
		WebSocketEventHundler.WebSocketServer.sockets.remove(self)

	def received_message(self, message):

		print("WebSocketEventHundler received_message")
		self.WebSocketServer.listener_received_message(message)

# Server class
class WebSocketServer():

	def __init__(self):

		self.host = "localhost"
		self.port = 8137

		self.wserver = None
		self.wserver_app = None
		self.wserver_thread = None
				
		self.sockets = []

		self.listener_received_message = None

	def start_server(self):

		print("WebSocketServer start_server")

		if self.wserver:
			return False

		# initialize sever
		self.wserver_app = WebSocketWSGIApplicationEx(server = self, handler_cls = WebSocketEventHundler)

		self.wserver = WSGIServer((self.host, self.port), WebSocketWSGIRequestHandler)
		self.wserver.set_app(self.wserver_app)
		self.wserver.initialize_websockets_manager()

		# create WebSocket thread
		self.wserver_thread = threading.Thread(target = self.wserver.serve_forever)
		self.wserver_thread.daemon = True

		# start thread
		WebSocketEventHundler.WebSocketServer = self
		self.wserver_thread.start()

		print("WebSocketServer start_server end")

		return True

	def stop_server(self):

		print("WebSocketServer stop_server")

		if not self.wserver:
			return False

		# shut down server
		self.wserver.shutdown()
		self.wserver_thread.join()

		# clear states
		self.wserver = None
		self.wserver_app = None
		self.wserver_thread = None

		print("WebSocketServer stop_server end")
		return True

	# event listeners
	def set_listener(self, started_listener_function, message_listener_function):

		self.listener_started = started_listener_function
		self.listener_received_message = message_listener_function

	# command functions
	def broadcast(self, message):

		for socket in self.sockets:
			socket.send(message)


if __name__ == "__main__":
    register()
